package com.backend.controller;

import com.backend.pojo.Result;
import com.backend.pojo.User;
import com.backend.service.UserService;
import com.backend.utils.JwtUtil;
import com.backend.utils.Md5Util;
import com.backend.utils.ThreadLocalUtil;
import jakarta.validation.constraints.Pattern;
import org.hibernate.validator.constraints.URL;
import org.springframework.beans.factory.annotation.Autowired;
import org.springframework.data.redis.core.StringRedisTemplate;
import org.springframework.data.redis.core.ValueOperations;
import org.springframework.util.StringUtils;
import org.springframework.validation.annotation.Validated;
import org.springframework.web.bind.annotation.*;

import java.time.Duration;
import java.util.Map;

import static com.backend.interceptor.LoginInterceptor.AUTHORIZATION_HEADER;

@RestController
@RequestMapping("/user")
@Validated
public class UserController {

        @Autowired
        private UserService userService;

        @Autowired
        private StringRedisTemplate stringRedisTemplate;

        @PostMapping("/register")
        public Result<User> register(@Pattern(regexp = "^\\S{5,16}$") final String username, @Pattern(regexp = "^\\S{5,16}$") final String password) {
            final User existingUser = userService.findByUsername(username);
            if (existingUser == null) {
                userService.register(username, password);
                return Result.success();
            } else {
                return Result.error("用户名已被占用");
            }
        }

        @PostMapping("/login")
        public Result<String> login(@Pattern(regexp = "^\\S{5,16}$") final String username, @Pattern(regexp = "^\\S{5,16}$") final String password) {
            final User existingUser = userService.findByUsername(username);
            if (existingUser == null) {
                return Result.error("用户不存在");
            }
            if (Md5Util.getMD5String(password).equals(existingUser.getPassword())) {
                final Map<String, Object> map = Map.of("id", existingUser.getId(),
                        "username", existingUser.getUsername());
                final String token = JwtUtil.genToken(map);
                // 保存token到redis
                final ValueOperations<String, String> operations = stringRedisTemplate.opsForValue();
                // 设置token过期时间为12小时
                operations.set(token, token, Duration.ofHours(12));
                // 返回 JWT token
                return Result.success(token);
            } else {
                return Result.error("密码错误");
            }
        }

        @GetMapping("/userInfo")
        public Result<User> userInfo() {

            final Map<String, Object> map = ThreadLocalUtil.get();
            final String username = (String) map.get("username");
            final User user = userService.findByUsername(username);
            return Result.success(user);
        }

        @PutMapping("/update")
        public Result<String> update(@RequestBody @Validated final User user) {
            final Map<String, Object> map = ThreadLocalUtil.get();
            final Integer id = (Integer) map.get("id");
            if (user.getId().equals(id)) {
                userService.update(user);
                return Result.success();
            } else {
                return Result.error("非本人id");
            }
        }



        @PatchMapping("/updatePwd")
        public Result<String> updatePwd(@RequestBody Map<String, String> params, @RequestHeader(AUTHORIZATION_HEADER) final String token) {
            final String oldPwd = params.get("old_pwd");
            final String newPwd = params.get("new_pwd");
            final String rePwd = params.get("re_pwd");
            if (!StringUtils.hasLength(oldPwd) || !StringUtils.hasLength(newPwd) || !StringUtils.hasLength(rePwd)) {
                return Result.error("缺少必要的参数");
            }

            final Map<String, Object> map = ThreadLocalUtil.get();
            final String username = (String) map.get("username");
            final User user = userService.findByUsername(username);
            if (!rePwd.equals(newPwd)) {
                return Result.error("两次结果不一样");
            }
            if (!user.getPassword().equals(Md5Util.getMD5String(oldPwd))) {
                return Result.error("密码填写不正确");
            }

            userService.updatePwd(newPwd);
            // 删除token
            final ValueOperations<String, String> operations = stringRedisTemplate.opsForValue();
            operations.getOperations().delete(token);

            return Result.success();
        }
    }
